<?php

namespace GlueApps\GluePHP\Action;

use GlueApps\GluePHP\AbstractApp;
use GlueApps\GluePHP\Action\AbstractAction;
use GlueApps\GluePHP\Component\AbstractComponent;
use GlueApps\GluePHP\Component\Sidebar;
use GlueApps\GluePHP\Component\Model\Model;

/**
 * @author Andy Daniel Navarro Taño <andaniel05@gmail.com>
 */
class AppendAction extends AbstractAction
{
    protected $app;
    protected $parent;
    protected $child;
    protected $render;

    public function __construct(AbstractApp $app, AbstractComponent $parent, AbstractComponent $child, bool $render = true)
    {
        $this->app = $app;
        $this->parent = $parent;
        $this->child = $child;
        $this->render = $render;

        $html = $render ?
            AbstractComponent::containerView(
                $child->getId(),
                $child->html()
            ) : null;

        $html = null;
        if ($render) {
            if ($child instanceof Sidebar) {
                $html = $child->html();
            } else {
                $html = AbstractComponent::containerView(
                    $child->getId(),
                    $child->html()
                );
            }
        }

        $frontProcessors = [];
        foreach ($child->processors() as $class) {
            if (! $app->hasProcessorClass($class)) {
                $app->registerProcessorClass($class);
            }

            $frontProcessors[] = $app->getFrontProcessorClass($class);
        }

        parent::__construct([
            'parentId'   => $parent->getId(),
            'childId'    => $child->getId(),
            'html'       => $html,
            'strModel'   => Model::getJavaScriptModelObject($child),
            'processors' => $frontProcessors,
        ]);
    }

    public static function handlerScript(): string
    {
        return <<<JAVASCRIPT

    var parent = app.getComponent(data.parentId);

    if ('string' === typeof(data.html)) {
        var childElement = document.createElement('div');
        childElement.innerHTML = data.html;
        childElement = childElement.firstChild;
        parent.childrenElement.append(childElement);
    } else {
        var childElement = parent.element.querySelector('#gphp-' + data.childId);
    }

    var model = null;
    var str = 'model = ' + data.strModel + ';';
    eval(str);
    var child = new GluePHP.Component(data.childId, app, model, childElement);
    parent.addComponent(child);

    data.processors.forEach(function(id) {
        if ('function' === typeof(app.processors[id])) {
            app.processors[id](child);
        }
    });

JAVASCRIPT;
    }

    public function send(bool $value = true): void
    {
        parent::send($value);

        if ($this->app instanceof AbstractApp && $this->app->inProcess()) {
            foreach ($this->child->getEventRecord() as $name => $data) {
                $this->app->act(new RegisterEventAction("{$this->child->getId()}.{$name}", $data));
            }
        }
    }
}
